 /**
  *
  * Program name: EmptyQ
  *
  * Description: Simple C program to remove all messages from a
  *              queue.
  *
      DEFINE SERVICE ('Empty_NULL.Q') +
             DESCR('Continuously empty the NULL.Q') +
             STARTCMD('F:\eclipse\workspace\MQDrain\Release\emptyq.exe') +
             STARTARG('NULL.Q MQWT2') +
             STOPCMD(' ') +
             STOPARG(' ') +
             STDOUT('F:\eclipse\workspace\MQDrain\Release\stdout.log') +
             STDERR('F:\eclipse\workspace\MQDrain\Release\stderr.log') +
             CONTROL(STARTONLY) +
             SERVTYPE(SERVER) +
             REPLACE
  *
  **/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>

     /* includes for MQI  */
 #include <cmqc.h>

#define LOG_RECORD_FORMAT_STRING "%d/%02d/%02d %02d:%02d:%02d %s Line#%0.5d: %s\n"

/*
 * defines used for Log functions
 */
#define LOG_TYPE_INFO         0   /* General informational log msg */
#define LOG_TYPE_WARNING      1   /* Warning messages */
#define LOG_TYPE_ERROR        2   /* Error messages - show stoppers */

/* prototypes  */
static void LogIt( int , int, char *);

int main(int argc, char **argv)
{

   /*   Declare MQI structures needed                                */
   MQOD     od = {MQOD_DEFAULT};    /* Object Descriptor             */
   MQMD     md = {MQMD_DEFAULT};    /* Message Descriptor            */
   MQGMO   gmo = {MQGMO_DEFAULT};   /* get message options           */
      /** note, sample uses defaults where it can **/

   MQHCONN  Hcon;                   /* connection handle             */
   MQHOBJ   Hobj;                   /* object handle                 */
   MQLONG   O_options;              /* MQOPEN options                */
   MQLONG   C_options;              /* MQCLOSE options               */
   MQLONG   CompCode;               /* completion code               */
   MQLONG   OpenCode;               /* MQOPEN completion code        */
   MQLONG   Reason;                 /* reason code                   */
   MQLONG   CReason;                /* reason code for MQCONN        */
   char     Buffer[2];              /* point to buffer               */
   MQLONG   buflen = 1;
   MQLONG   messlen;                /* message length received       */
   char     QMName[50];             /* queue manager name            */
   int      RemoveCount = 0;
   char     TempString[1024];

   LogIt( LOG_TYPE_INFO,  __LINE__, "EmptyQ starting" );
   if (argc < 2)
   {
      LogIt( LOG_TYPE_ERROR,  __LINE__, "Required parameter missing - queue name\n" );
      exit(99);
   }

   /******************************************************************/
   /*                                                                */
   /*   Create object descriptor for subject queue                   */
   /*                                                                */
   /******************************************************************/
   strcpy(od.ObjectName, argv[1]);
   QMName[0] = 0;   /* default */
   if (argc > 2)
     strcpy(QMName, argv[2]);

   /******************************************************************/
   /*                                                                */
   /*   Connect to queue manager                                     */
   /*                                                                */
   /******************************************************************/
   MQCONN(QMName,                  /* queue manager                  */
          &Hcon,                   /* connection handle              */
          &CompCode,               /* completion code                */
          &CReason);               /* reason code                    */

   /* report reason and stop if it failed     */
   if (CompCode == MQCC_FAILED)
   {
      sprintf( TempString,
               "MQCONN ended with reason code %d",
               CReason);
               
      LogIt( LOG_TYPE_ERROR,  __LINE__, TempString );
      exit( (int)CReason );
   }

   /******************************************************************/
   /*                                                                */
   /*   Open the named message queue for input; exclusive or shared  */
   /*   use of the queue is controlled by the queue definition here  */
   /*                                                                */
   /******************************************************************/

   if (argc > 3)
   {
     O_options = atoi( argv[3] );
      sprintf( TempString,
               "open  options are %d", 
               O_options);
               
      LogIt( LOG_TYPE_INFO,  __LINE__, TempString );
   }
   else
   {
     O_options = MQOO_INPUT_SHARED | MQOO_FAIL_IF_QUIESCING;
   }


   MQOPEN(Hcon,                      /* connection handle            */
          &od,                       /* object descriptor for queue  */
          O_options,                 /* open options                 */
          &Hobj,                     /* object handle                */
          &OpenCode,                 /* completion code              */
          &Reason);                  /* reason code                  */

   /* report reason, if any; stop if failed      */
   if (Reason != MQRC_NONE)
   {
      sprintf( TempString,
               "MQOPEN ended with reason code %d", 
               Reason);
               
      LogIt( LOG_TYPE_ERROR,  __LINE__, TempString );
   }

   if (OpenCode == MQCC_FAILED)
   {
      LogIt( LOG_TYPE_ERROR,  __LINE__, "unable to open queue for input" );
   }

   CompCode = OpenCode;       /* use MQOPEN result for initial test  */

   gmo.Options = MQGMO_FAIL_IF_QUIESCING + MQGMO_ACCEPT_TRUNCATED_MSG + MQGMO_WAIT;
   gmo.WaitInterval = MQWI_UNLIMITED;

   while (CompCode != MQCC_FAILED)
   {
     Buffer[0] = '\0';
     Buffer[1] = '\0';

     memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
     memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));

     md.Encoding       = MQENC_NATIVE;
     md.CodedCharSetId = MQCCSI_Q_MGR;

     MQGET(Hcon,                /* connection handle                 */
           Hobj,                /* object handle                     */
           &md,                 /* message descriptor                */
           &gmo,                /* get message options               */
           buflen,              /* Buffer length                    */
           Buffer,              /* pointer to message buffer         */
           &messlen,            /* message length                    */
           &CompCode,           /* completion code                   */
           &Reason);            /* reason code                       */

     /* report reason, if any     */
     if (Reason != MQRC_NONE)
     {
       if (Reason == MQRC_NO_MSG_AVAILABLE)
       {                         /* special report for normal end    */
//          printf("no more messages\n");
       }
       else                      /* general report for other reasons */
       {
         // printf("MQGET ended with reason code %d\n", Reason);

         /*   treat truncated message as a OK  */
         if (Reason == MQRC_TRUNCATED_MSG_ACCEPTED)
         {
           CompCode = MQCC_OK;
           RemoveCount++;
         }
       }
     }
     else 
        RemoveCount++;

   }

   sprintf( TempString,
            "%d messages removed from queue: %s on %s",
            RemoveCount, 
            argv[1], 
            argv[2]);
            
   LogIt( LOG_TYPE_INFO,  __LINE__, TempString );

   /******************************************************************/
   /*   Close the source queue (if it was opened)                    */
   /******************************************************************/
   if (OpenCode != MQCC_FAILED)
   {
     if (argc > 4)
     {
       C_options = atoi( argv[4] );
       
       sprintf( TempString,
                "close options are %d", 
                C_options);
               
       LogIt( LOG_TYPE_INFO,  __LINE__, TempString );
       
     }
     else
     {
       C_options = MQCO_NONE;        /* no close options             */
     }

     MQCLOSE(Hcon,                    /* connection handle           */
             &Hobj,                   /* object handle               */
             C_options,
             &CompCode,               /* completion code             */
             &Reason);                /* reason code                 */

     /* report reason, if any     */
     if (Reason != MQRC_NONE)
     {
         sprintf( TempString,
                  "MQCLOSE ended with reason code %d", 
                  Reason);
                  
         LogIt( LOG_TYPE_ERROR,  __LINE__, TempString );
     }
   }

   /******************************************************************/
   /*                                                                */
   /*   Disconnect from MQM if not already connected                 */
   /*                                                                */
   /******************************************************************/
   if (CReason != MQRC_ALREADY_CONNECTED )
   {
     MQDISC(&Hcon,                     /* connection handle          */
            &CompCode,                 /* completion code            */
            &Reason);                  /* reason code                */

     /* report reason, if any     */
     if (Reason != MQRC_NONE)
     {
         sprintf( TempString,
                  "MQDISC ended with reason code %d", 
                  Reason);
                  
         LogIt( LOG_TYPE_ERROR,  __LINE__, TempString );
     }
   }

   /******************************************************************/
   /*                                                                */
   /* END OF AMQSGET0                                                */
   /*                                                                */
   /******************************************************************/
   LogIt( LOG_TYPE_INFO,  __LINE__, "EmptyQ ending\n" );
   return(0);
}


static void LogIt( int         severity,
                   int         line,
                   char       *msg )
{
   char            cSeverity[10];
   struct tm*      CurrentTime;
   time_t          LongTime;

   // ---------------------------------------------------------------
   // Change severity level to text description.
   // ---------------------------------------------------------------

   switch( severity )
   {
      case 0:
       strcpy( cSeverity, "INFO   " );
       break;
      case 1:
       strcpy( cSeverity, "WARNING" );
       break;
      case 2:
       strcpy( cSeverity, "ERROR  " );
       break;
      default:
       strcpy( cSeverity, "ERROR+ " );
       break;
    }

   // ---------------------------------------------------------------
   // Get the current (local) time.
   // ---------------------------------------------------------------

   LongTime = time(NULL);
   CurrentTime = localtime(&LongTime);

   printf( LOG_RECORD_FORMAT_STRING,
            (CurrentTime->tm_year + 1900),
            (CurrentTime->tm_mon + 1),
            CurrentTime->tm_mday,
            CurrentTime->tm_hour,
            CurrentTime->tm_min,
            CurrentTime->tm_sec,
            cSeverity,
            line,
            msg );

   return;
}
